/*
 * Freedom Desktop
 * Copyright 1994 by Freedom Software
 *
 * Freedom Software retains all rights to Freedom Desktop (hereafter Software)
 * in binary and in source code form.
 *
 * The commercial use of this Software shall be governed by a separate License
 * agreement. Any individual or institution wishing to make commercial use of
 * the Software must sign a license agreement with Freedom Software. In such
 * cases, the Licensee agrees to abide by the terms contained in the License
 * Agreement and not those contained in this document. Examples of commercial
 * use include (without limitation): (i) integration of the Software (source
 * code form), in whole or in part, into a commercial product sold by or on
 * on behalf of the Licensee; (ii) distribution of the Software (binary form or
 * source code form) in combination with a commercial product sold by or on
 * behalf of the Licensee.
 *
 * Freedom Software (Licensor) grants you (Licensee) a license: (i) to use,
 * copy and make changes and improvements to this Software for licensee's
 * internal business purposes; (ii) to use, copy, and distribute this Software
 * or the derivative works provided that the copyright notice and this
 * permission notice appear on all copies and that NO CHARGE is associated
 * with such copies. However, if Licensee distributes any derivative work
 * based on the Software, then Licensee shall (i) notify Licensor in writing
 * (ii) clearly state that such derivative work is a modified and not the
 * original Freedom Desktop distributed by Freedom Software (iii) publish
 * the corresponding machine-readable source code or information as to
 * where it may be obtained. Each time Licensee redistribute the Software
 * or any derivative work, the recipient automatically agrees to abide
 * by the same terms as the Licensee. Licensee may not impose terms
 * more restrictive than the terms granted herein.
 *
 * By using, copying, modifying or distributing this Software (or any
 * derivative work based on this Software) Licensee indicates acceptance
 * of the terms and conditions set forth in this License.
 *
 * Licensor reserves the right to terminate this License immediately on written
 * notice, for material breach by the Licensee.
 *
 * FREEDOM SOFTWARE DISCLAIMS ALL WARRANTIES EXPRESS OR IMPLIED WITH REGARD
 * TO THIS SOFTWARE INCLUDING BUT NOT LIMITED TO ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND  FITNESS,  IN  NO  EVENT  SHALL LICENSOR BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE
 */

#ifndef SIGNAL_HANDLER_H
#define SIGNAL_HANDLER_H

#include "Rt.h"
#include <signal.h>

/*
 * Macros and definitions
 */

#define RtMAX_SIGNAL 50

typedef struct _SignalHandlerClassPart {
    int foo;  
} SignalHandlerClassPart;

typedef struct _SignalHandlerClassRec {
    ObjCoreClassPart core_class;
    SignalHandlerClassPart form_class;
} SignalHandlerClassRec;

extern  SignalHandlerClassRec   signalHandlerClassRec;
extern  ObjClass signalHandlerObjClass;


typedef struct _SignalHandlerPart {
   struct sigaction signal_table[RtMAX_SIGNAL]; 
   sigset_t signal_mask;
   sigset_t old_mask;
   sigset_t pending_mask;
   int current_signal;
   int is_pending;  
} SignalHandlerPart;

/*
 * Signal Handler record
 */
 
typedef struct _SignalHandlerRec {
   ObjCorePart core;
   MessagePart message;
   SignalHandlerPart signal;
} SignalHandlerRec;

typedef SignalHandlerRec        *SignalHandlerObj;
typedef SignalHandlerClassRec   *SignalHandlerObjClass;

/*
 * Resources
 */
 
#ifdef _NO_RESOURCE
#define RtNsigMask (((SignalHandlerObj)(new))->signal.signal_mask)
#define RtNsigCurrent (((SignalHandlerObj)(new))->signal.current_signal)
#define RtNsigIsPending (((SignalHandlerObj)(new))->signal.is_pending)
#else
#define RtNsigMask 	"sigMask"
#define RtNsigCurrent	"sigCurrent" 
#define RtNsigIsPending	"sigIsPending" 
#endif

/*
 * Messages
 */
 
#define RtSIG_ADD_SET   2400     /* Add signal to the signal mask   */
#define RtSIG_BLOCK     2401     /* Update the process signal mask  */
#define RtSIG_PENDING   2402     /* Get the mask of pending signals */
                                 /* and determine if a specific     */
                                 /* signal is pending               */
#define RtSIG_MASK_RESTORE  2403 /* Restore the signal mask	    */                                
#define RtSIG_IGNORE	2404	 /* Ignore signal		    */
#define RtSIG_RESTORE	2405	 /* Restore the signal action       */                                
#endif
