/*
 * Freedom Plug & Play reusable toolkit (Rt)
 * Copyright 1994 by Freedom Software
 *
 * Freedom Software retains all rights to Freedom Desktop (hereafter Software)
 * in binary and in source code form.
 *
 * The commercial use of this Software shall be governed by a separate License
 * agreement. Any individual or institution wishing to make commercial use of
 * the Software must sign a license agreement with Freedom Software. In such
 * cases, the Licensee agrees to abide by the terms contained in the License
 * Agreement and not those contained in this document. Examples of commercial
 * use include (without limitation): (i) integration of the Software (source
 * code form), in whole or in part, into a commercial product sold by or on
 * on behalf of the Licensee; (ii) distribution of the Software (binary form or
 * source code form) in combination with a commercial product sold by or on
 * behalf of the Licensee.
 *
 * Freedom Software (Licensor) grants you (Licensee) a license: (i) to use,
 * copy and make changes and improvements to this Software for licensee's
 * internal business purposes; (ii) to use, copy, and distribute this Software
 * or the derivative works provided that the copyright notice and this
 * permission notice appear on all copies and that NO CHARGE is associated
 * with such copies. However, if Licensee distributes any derivative work
 * based on the Software, then Licensee shall (i) notify Licensor in writing
 * (ii) clearly state that such derivative work is a modified and not the
 * original Freedom Desktop distributed by Freedom Software (iii) publish
 * the corresponding machine-readable source code or information as to
 * where it may be obtained. Each time Licensee redistribute the Software
 * or any derivative work, the recipient automatically agrees to abide
 * by the same terms as the Licensee. Licensee may not impose terms
 * more restrictive than the terms granted herein.
 *
 * By using, copying, modifying or distributing this Software (or any
 * derivative work based on this Software) Licensee indicates acceptance
 * of the terms and conditions set forth in this License.
 *
 * Licensor reserves the right to terminate this License immediately on written
 * notice, for material breach by the Licensee.
 *
 * FREEDOM SOFTWARE DISCLAIMS ALL WARRANTIES EXPRESS OR IMPLIED WITH REGARD
 * TO THIS SOFTWARE INCLUDING BUT NOT LIMITED TO ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND  FITNESS,  IN  NO  EVENT  SHALL LICENSOR BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE
 */

/*
 * Multiplexer Object
 */
 
#include "Multiplexer.h"

void MXInitialize ();
void MXActivate ();
void MXRealize ();
void MXDestroy ();
void MXSetValues ();
void MXProcessMessage ();

MultiplexerClassRec multiplexerClassRec = {
  {
    (ObjClass) &broadCasterClassRec,	/* superclass		*/
    "Multiplexer",		/* class name		*/
    NULL,	
    0,
    NULL,			/* resource table	*/
    0,				/* number of resources  */
    /* object size              */  sizeof(MultiplexerRec),
    MXInitialize,
    MXRealize,
    NULL,
    MXSetValues,
    MXActivate,
    MXDestroy,
    MXProcessMessage,
    NULL,
    "Rt1.00",
    NULL
  },
  {
  0,
  }
  
};

ObjClass multiplexerObjClass = (ObjClass) &multiplexerClassRec; 

/*
 * MXSetValues: set object resources
 */
 
void MXSetValues (current, new)
MultiplexerObj current, new;
{
   /* Check - This should be done by the superclass */
   RtUpdateString (current->core.object_name, new->core.object_name);
}

/*
 * MXInitialize: initialize the object
 */

void MXInitialize (new)
MultiplexerObj new;
{
#ifdef DEBUG
   new->core.object_name = strdup ("Multiplexer");
#endif
	
}

/*
 * MXActivate: activate object
 */
 
void MXActivate (new)
MultiplexerObj new;
{
}

/*
 * MXRealize: realize object
 */
 
void MXRealize (new)
MultiplexerObj new;
{
}

/*
 * MXDestroy: destroy object
 */
 
void MXDestroy (obj)
MultiplexerObj obj;
{
   /* destroy the broadcaster portion of the object */
   (*broadCasterObjClass->core_class.destroy) (obj);


   /* free the object itself */
   if (obj->core.object_class == multiplexerObjClass) {
      /* destroy core portion of the object */
      RtDestroyString (obj->core.object_name);   
      free (obj); 
   }  
}

/*
 * MXProcessMessage: process object messages
 */
 
void MXProcessMessage (obj, data, client)
MultiplexerObj obj;
void *data;
void *client;
{
MessageObj msg;
char *content;
int cnt;
int selection;
Obj receiver;
int message_id;

   msg = (MessageObj) data;
   RtGetValue (msg, RtNmsgId, &message_id);   
   RtGetValue (msg, RtNmsgContent, &content);         

   switch (message_id) {
      case RtMX_SELECTED:
	RtSetValue (obj, RtNbcReceiverSelected, (int) content);
	RtSetValue (obj, RtNbcMultiplexerMode, 1);
#ifdef DEBUG
	fprintf (stderr, "MXProcessMessage (RtMX_SELECTED) %d\n",
			(int) content);	        
#endif
        break;          
      case RtADD_RECEPTOR: /* Add a new receptor */
      case RtRESET:
        /* Invoke the superclass method to add a new receptor */
        (*broadCasterObjClass->core_class.process_message) (obj, 
                        (char *) data, NULL);
        break;            
      default:
        (*broadCasterObjClass->core_class.process_message) (obj, 
                        (char *) data, NULL);
        break;
   }
   
}



