/*
 * Freedom Desktop
 * Copyright 1994 by Freedom Software
 *
 * Freedom Software retains all rights to Freedom Desktop (hereafter Software)
 * in binary and in source code form.
 *
 * The commercial use of this Software shall be governed by a separate License
 * agreement. Any individual or institution wishing to make commercial use of
 * the Software must sign a license agreement with Freedom Software. In such
 * cases, the Licensee agrees to abide by the terms contained in the License
 * Agreement and not those contained in this document. Examples of commercial
 * use include (without limitation): (i) integration of the Software (source
 * code form), in whole or in part, into a commercial product sold by or on
 * on behalf of the Licensee; (ii) distribution of the Software (binary form or
 * source code form) in combination with a commercial product sold by or on
 * behalf of the Licensee.
 *
 * Freedom Software (Licensor) grants you (Licensee) a license: (i) to use,
 * copy and make changes and improvements to this Software for licensee's
 * internal business purposes; (ii) to use, copy, and distribute this Software
 * or the derivative works provided that the copyright notice and this
 * permission notice appear on all copies and that NO CHARGE is associated
 * with such copies. However, if Licensee distributes any derivative work
 * based on the Software, then Licensee shall (i) notify Licensor in writing
 * (ii) clearly state that such derivative work is a modified and not the
 * original Freedom Desktop distributed by Freedom Software (iii) publish
 * the corresponding machine-readable source code or information as to
 * where it may be obtained. Each time Licensee redistribute the Software
 * or any derivative work, the recipient automatically agrees to abide
 * by the same terms as the Licensee. Licensee may not impose terms
 * more restrictive than the terms granted herein.
 *
 * By using, copying, modifying or distributing this Software (or any
 * derivative work based on this Software) Licensee indicates acceptance
 * of the terms and conditions set forth in this License.
 *
 * Licensor reserves the right to terminate this License immediately on written
 * notice, for material breach by the Licensee.
 *
 * FREEDOM SOFTWARE DISCLAIMS ALL WARRANTIES EXPRESS OR IMPLIED WITH REGARD
 * TO THIS SOFTWARE INCLUDING BUT NOT LIMITED TO ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND  FITNESS,  IN  NO  EVENT  SHALL LICENSOR BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE
 */

/*
 * Generic object
 */

#include "ToggleWrapper.h"
 
void TOGGLEWInitialize ();
void TOGGLEWRealize ();
void TOGGLEWDestroy ();
void TOGGLEWSetValues ();
void TOGGLEWProcessMessage ();
static void toggle_value ();

static RtResource resources[] = {
   {RtNtoggleSet, NULL, RtINT_TYPE, sizeof(int), 
   RtOffset(XmToggleWrapperObj,toggle_wrapper.set), 0, NULL, 0, NULL },   
};

XmToggleWrapperClassRec xmToggleWrapperClassRec = {
  {
    (ObjClass) &xmLabelWrapperClassRec, /* superclass		*/
    "Toggle Wrapper",			/* class name		*/
    NULL,	
    0,
    resources,			/* resource table	*/
    RtNumber (resources),	/* number of resources  */
    /* object size              */  sizeof(XmToggleWrapperRec),
    TOGGLEWInitialize,
    TOGGLEWRealize,
    NULL,
    TOGGLEWSetValues,
    NULL,
    TOGGLEWDestroy,
    TOGGLEWProcessMessage,
    "toggle",
    "Rt1.00",
    NULL
  },
  {
  0,
  }
  
};

ObjClass xmToggleWrapperObjClass = (ObjClass) &xmToggleWrapperClassRec; 

/*
 * TOGGLEWSetValues: set object resources
 */
 
void TOGGLEWSetValues (current, new)
XmToggleWrapperObj current, new;
{
   /* Check - This should be done by the superclass */
   
   if (current->core.object_name != new->core.object_name)
        RtUpdateString (current->core.object_name, 
        	new->core.object_name);

   if (current->toggle_wrapper.set != new->toggle_wrapper.set)
   	if (new->toggle_wrapper.set)
   	   XmToggleButtonSetState (new->widget.widget,
   		True, False);
   	else
   	   XmToggleButtonSetState (new->widget.widget,
   		False, False);
   	
}


/*
 * TOGGLEWInitialize: initialize object
 */

void TOGGLEWInitialize (obj)
XmToggleWrapperObj obj;
{
#ifdef DEBUG
   obj->core.object_name = strdup ("Generic Object");
#endif
}

/*
 * TOGGLEWRealize: realize object
 */
 
void TOGGLEWRealize (obj)
XmToggleWrapperObj obj;
{
   if (obj->widget.widget) {
   	XtAddCallback (obj->widget.widget, XmNvalueChangedCallback, 
        	toggle_value, obj);
   }
   
}


/*
 * TOGGLEWDestroy: destroy object
 */
 
void TOGGLEWDestroy (obj)
XmToggleWrapperObj obj;
{
}

void TOGGLEWProcessMessage (obj, data, client)
XmToggleWrapperObj obj;
void *data;
char *client;
{
MessageObj msg;
char *content;
int flags;
int message_id;

   msg = (MessageObj) data;
   RtGetValue (msg, RtNmsgId, &message_id);   
   RtGetValue (msg, RtNmsgContent, &content);   
   RtGetValue (msg, RtNmsgFlags, &flags);   
   
   /* ignore the function paramenter  		*/
   /* get the client info from the message 	*/
   RtGetValue (msg, RtNmsgClient, &client); 

   switch (message_id) {
      case RtSET_TOGGLE_ON:
        if (!obj->widget.widget)
           return;
   	XmToggleButtonSetState (obj->widget.widget,
   	   	True, True);
   	break;
      case RtSET_TOGGLE_OFF:
        if (!obj->widget.widget)
           return;
	XmToggleButtonSetState (obj->widget.widget,
   	   	False, True);
   	break;
      default:
#ifdef DEBUG
   	fprintf (stderr, 
   	"TOGGLEWProcessMessage: I don't know how to process this message\n");
#endif
	/* Invoke the superclass to process the message */
	RtSuperclassProcessMessage (xmToggleWrapperObjClass,
		obj, data, client); 
        break;
   }
}

/*
 * toggle_value: toggle value callback
 */
 
static void toggle_value (widget, client_data, cbs)
Widget widget;
XtPointer client_data;
XmToggleButtonCallbackStruct *cbs;
{
XmToggleWrapperObj obj = (XmToggleWrapperObj) client_data;

   if (!obj)
     	return;
     
   obj->toggle_wrapper.set = cbs->set;
   
   if (cbs->set) {   
   	RtSetValue (stdmsg, RtNmsgId, RtTOGGLE_ON);
   } else {
   	RtSetValue (stdmsg, RtNmsgId, RtTOGGLE_OFF);
   }
        
   /* Invoke the superclass method to broadcast the message */
   (*broadCasterObjClass->core_class.process_message) (obj, 
                        (char *) stdmsg);
	     
}
