
/*
 * Freedom Desktop
 * Copyright 1994 by Freedom Software
 *
 * Freedom Software retains all rights to Freedom Desktop (hereafter Software)
 * in binary and in source code form.
 *
 * The commercial use of this Software shall be governed by a separate License
 * agreement. Any individual or institution wishing to make commercial use of
 * the Software must sign a license agreement with Freedom Software. In such
 * cases, the Licensee agrees to abide by the terms contained in the License
 * Agreement and not those contained in this document. Examples of commercial
 * use include (without limitation): (i) integration of the Software (source
 * code form), in whole or in part, into a commercial product sold by or on
 * on behalf of the Licensee; (ii) distribution of the Software (binary form or
 * source code form) in combination with a commercial product sold by or on
 * behalf of the Licensee.
 *
 * Freedom Software (Licensor) grants you (Licensee) a license: (i) to use,
 * copy and make changes and improvements to this Software for licensee's
 * internal business purposes; (ii) to use, copy, and distribute this Software
 * or the derivative works provided that the copyright notice and this
 * permission notice appear on all copies and that NO CHARGE is associated
 * with such copies. However, if Licensee distributes any derivative work
 * based on the Software, then Licensee shall (i) notify Licensor in writing
 * (ii) clearly state that such derivative work is a modified and not the
 * original Freedom Desktop distributed by Freedom Software (iii) publish
 * the corresponding machine-readable source code or information as to
 * where it may be obtained. Each time Licensee redistribute the Software
 * or any derivative work, the recipient automatically agrees to abide
 * by the same terms as the Licensee. Licensee may not impose terms
 * more restrictive than the terms granted herein.
 *
 * By using, copying, modifying or distributing this Software (or any
 * derivative work based on this Software) Licensee indicates acceptance
 * of the terms and conditions set forth in this License.
 *
 * Licensor reserves the right to terminate this License immediately on written
 * notice, for material breach by the Licensee.
 *
 * FREEDOM SOFTWARE DISCLAIMS ALL WARRANTIES EXPRESS OR IMPLIED WITH REGARD
 * TO THIS SOFTWARE INCLUDING BUT NOT LIMITED TO ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND  FITNESS,  IN  NO  EVENT  SHALL LICENSOR BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE
 */

#include <stdio.h>
#include <stdlib.h>
#ifdef SUNOS
#include <strings.h>
#endif
#include "etc.h"

char *_trashpath;
char *_editor;
char *_homepath;
char *_freedompath;
char *_treepath; 
char *_iconspath;
char *_keypath;
char *_lgroupspath; 
char *_trashname = "trash";
char *_treename = ".freedom";
char *_groupsname = "groups";
char *_maingroup = "Main";
char *_lfreedompath;

static char *_basename(name) 
char *name;
{
        register char *cp;

        cp = rindex(name, '/');
        return(cp ? cp + 1 : name);
}



int get_env_var ()
{
     if (!(_homepath = getenv("HOME"))) { 
        fprintf (stderr, "get_env_var: Couldn't find value of variable HOME\n"); 
        return (0);
     }
     _homepath = strdup (_homepath);

     if (!_homepath) {
	fprintf (stderr, "get_env_var: _strdup failed\n");
	return (0);
     }

     _lfreedompath = _dircat (_homepath, _treename);

     if (!_lfreedompath) {
	fprintf (stderr, "get_env_var: _dircat failed\n");
	return (0);
     }

     _trashpath = _dircat (_lfreedompath, _trashname);
     if (!_trashpath) {
	fprintf (stderr, "get_env_var: _dircat failed\n");
	return (0);
     }

     if ((_freedompath = getenv("FREEDOMDIR")) == NULL) { 
           fprintf (stderr, "get_env_var: variable FREEDOMDIR not set\n");
	   return (0);
     }

     _iconspath = _dircat (_freedompath, "icons");

     if (!_iconspath) {
        fprintf (stderr, "get_env_var: _dircat failed\n");
        return (0);
     }

     _keypath = _dircat (_freedompath, "key");

     if (!_keypath) {
        fprintf (stderr, "get_env_var: _dircat failed\n");
        return (0);
     }

     if (!(_editor = getenv ("EDITOR")))  
	_editor = strdup ("vi");
     else {
	if (strchr (_editor, '/')) {
	  int status;

	   if ((status = _fexecutable (_editor)) < 0) {
	   	fprintf (stderr, "Warning: EDITOR has been set to %s\n",
	   		_editor);
	   	perror (_editor);
	   } else if (status == 0)
	   	fprintf (stderr, "Warning: EDITOR (%s) is not readable and/or executable\n",
			_editor);	   	
	   
	}
		
	_editor = strdup (_editor);
     }

     if (!_editor) {
        fprintf (stderr, "get_env_var: _strdup failed\n");
        return (0);
     }

     return (1);

}
